import requests
import json
import pandas as pd
from Series_Time import *

def getData(date):  
    url =  'http://192.168.0.35:5060/ldat'
    param =  {
        "id": "10751",
        "reqdate": date
         # 04-06-2023
    }   
    try:
        response = requests.post(url,json=param, verify=False, timeout=20)
    
    except:
        print('server busy')
    dat = response.text
    dt = json.loads(dat)
    return dt

def get_live_data(yesterday,today,interval=15):
    t = calculate_completed_intervals(interval)
    if t == 0:
        live_data = pd.concat([yesterday.iloc[:, -2:], today.iloc[:, :1]], axis=1)
        return live_data
    elif t == 1:
        live_data = pd.concat([yesterday.iloc[:, -1:], today.iloc[:, :2]], axis=1)
        return live_data
    elif t == 2:
        live_data = today[:3]
        return live_data
    elif t >= 3:
        live_data = today.iloc[:,t-2:t+1]
        return live_data
    
def get_pred_data(yesterday,pred_data,interval=15):
    t = calculate_completed_intervals(interval)
    if t == 0:
        pred = pd.concat([yesterday.iloc[:, -2:], pred_data.iloc[:, :1]], axis=1)
        return pred
    elif t == 1:
        pred = pd.concat([yesterday.iloc[:, -1:], pred_data.iloc[:, :2]], axis=1)
        return pred
    elif t == 2:
        pred = pred_data.iloc[:,0:3]
        return pred
    elif t >= 3:
        pred = pred_data.iloc[:,t-2:t+1]
        return pred
    
def get_next_pred(pred,next_pred_intervals=8):
    t = calculate_completed_intervals(15)
    return pred.iloc[:,t:t+next_pred_intervals]    




import pandas as pd
import numpy as np

def MasterPredict(input_df,live_data,sample_data):
    output_values = []
    prev_day = input_df
    sample_data_columns = sample_data.columns 
    prev_day_values = prev_day.values
    live_data_values = live_data.values
    sample_data_values = sample_data.values.flatten()
    prev_day_err=(np.array(live_data)-prev_day_values)/prev_day_values
    prev_day_err = prev_day_err.mean()
    for i in list(sample_data_values):
        corr = (i+(i*prev_day_err))
        output_values.append(corr)
    
    output_df = pd.DataFrame([output_values],columns=sample_data_columns)
    return output_df


import pandas as pd
import numpy as np

def fill_missing_values(df):
    # Iterate over each row
    for row_index, row in df.iterrows():
        # Iterate over each column
        for col_index, value in enumerate(row):
            # Check if the value is empty
            if pd.isnull(value):
                # Find the closest non-empty values
                before = df.loc[row_index, df.columns[:col_index]][::-1].dropna()
                after = df.loc[row_index, df.columns[col_index+1:]].dropna()
                
                # Fill the missing value with the closest non-empty value
                if len(before) > 0 and len(after) > 0:
                    df.loc[row_index, df.columns[col_index]] = np.mean([before.iloc[0], after.iloc[0]])
                elif len(before) > 0:
                    df.loc[row_index, df.columns[col_index]] = before.iloc[0]
                elif len(after) > 0:
                    df.loc[row_index, df.columns[col_index]] = after.iloc[0]

                # If there are no non-empty values, set the missing value to 0
                else:
                    df.loc[row_index, df.columns[col_index]] = 0

    return df

def remove_outliers(df, low_percentile=25, high_percentile=75, outlier_threshold=1.5):
    cleaned_df = df.copy()
    removed_values = []
    for row_idx, row in cleaned_df.iterrows():
        low_threshold = np.percentile(row, low_percentile)
        high_threshold = np.percentile(row, high_percentile)
        iqr = high_threshold - low_threshold
        low_limit = low_threshold - outlier_threshold * iqr
        high_limit = high_threshold + outlier_threshold * iqr
        outliers = row[(row < low_limit) | (row > high_limit)]
        for col_name in outliers.index:
            removed_values.append({'Row': row_idx, 'Column': col_name, 'Value': row[col_name]})
            cleaned_df.loc[row_idx, col_name] = np.nan
    removed_values_df = pd.DataFrame(removed_values)
    return cleaned_df, removed_values_df


from datetime import datetime, timedelta

def get_previous_weekday_dates():
    current_date = datetime.now().date()
    weekday = current_date.weekday()

    previous_dates = []
    i = 1
    while len(previous_dates) < 3:
        delta = timedelta(days=i)
        previous_date = current_date - delta
        if previous_date.weekday() == weekday:
            previous_dates.append(previous_date.strftime("%d-%m-%Y"))
        i += 1

    # Inserting today's date at index 0
    today_date = current_date.strftime("%d-%m-%Y")
    previous_dates.insert(0, today_date)

    return previous_dates

def calculate_completed_intervals(n):
    current_time = datetime.now().time()
    total_minutes = current_time.hour * 60 + current_time.minute
    completed_intervals = total_minutes // n

    return completed_intervals


import pandas as pd
import pandas as pd
from datetime import datetime, timedelta

def next_series_data(df):
    # Create an empty DataFrame to store the adjusted averages
    adjusted_moving_df = pd.DataFrame()

    # Iterate over every 3 rows in the original DataFrame
    for i in range(0, len(df), 3):
        # Select the current three rows
        rows = df.iloc[i:i+3]

        # Calculate the adjusted averages for the current three rows
        moving_averages = []
        for column in df.columns:
            r1 = rows[column].iloc[0]
            r2 = rows[column].iloc[1]
            r3 = rows[column].iloc[2]

            # Calculate the initial average for the current column
            average = (r1 + r2 + r3) / 3

            # Adjust the average until the absolute difference is less than 5
            while abs(average - r3) >= 5:
                if average > r3:
                    average -= 0.01  # Decrease the average by 0.01
                else:
                    average += 0.01  # Increase the average by 0.01

            moving_averages.append(average)

        # Append the adjusted averages to the new DataFrame
        adjusted_moving_df = pd.concat([adjusted_moving_df, pd.DataFrame([moving_averages], columns=df.columns)], ignore_index=True)

    return adjusted_moving_df




def combine_json_data(json_files):
    combined_df = pd.DataFrame()

    for index, json_data in enumerate(json_files):
        data = json_data['data']['10751']
        df = pd.DataFrame(data)
        df['ts'] = pd.to_datetime(df['ts'])
        df['ts'] = df['ts'].dt.strftime('%H:%M')
        df = df.drop('status', axis=1)
        
        cols = df.columns.tolist()
        cols = ['ts'] + [col for col in cols if col != 'ts']
        df = df[cols]
        
        if combined_df.empty:
            combined_df = df[['ts', 'value']].rename(columns={'value': 'n-3'})
        else:
            column_name = 'n-' + str(len(json_files) - index)
            combined_df = pd.merge(combined_df, df[['ts', 'value']].rename(columns={'value': column_name}), on='ts')

    combined_df.set_index('ts', inplace=True)
    combined_df = combined_df.astype(float)
    combined_df = combined_df.T

    return combined_df

def prev_date():
    current_date = datetime.now().date()
    yesterday_date = current_date - timedelta(days=1)
    return yesterday_date.strftime("%d-%m-%Y")


from setuptools import setup

setup(
    name='ML-Algo',
    version='1.2.2',
    author='Mr Raj',
    author_email='arunraj14092002@gmail.com',
    description='A package for calculating Series Time Data',
    packages=['ML_Algo'],
    install_requires=['pandas'],
)